<?php
namespace AustraliaPost\Extensions\Business\Endpoints;

use AustraliaPost\Extensions\Business\API\Wrapper\Price;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Feature;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Item;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Request;
use AustraliaPost\Extensions\Business\Helpers\Utilities;

class Calculate_Prices_Endpoint extends Abstract_Endpoint {


	public function callback( $data )
	{
		if (!isset($data['shipments']) || !is_array($data['shipments'])) {
			$this->abort('shipments must be provided');
		}

		$order = $this->get_order($data);
		$from = $this->get_from_address();
		$to = $this->get_to_address($order);

		$priceRequest = new Price_Request();
		$priceRequest
			->setFromPostcode($from->getPostcode())
			->setFromCountry($from->getCountry())
			->setToPostcode($to->getPostcode())
			->setToCountry($to->getCountry());

		foreach ($data['shipments'] as $index => $shipment)
		{
			$cubicWeight = $this->cubicWeight($shipment);
			if ($shipment['product_id'] === 'B20' && max($cubicWeight, $shipment['weight']) > 5000) {
				$shipment['product_id'] = 'B21';
				$data['shipments'][$index]['product_id'] = 'B21';
			}

			if ($shipment['product_id'] === 'B30' && max($cubicWeight, $shipment['weight']) > 5000) {
				$shipment['product_id'] = 'B31';
				$data['shipments'][$index]['product_id'] = 'B31';
			}

			$priceItem = (new Price_Item(Utilities::accountType()))
				->setLength($shipment['length'] / 10)
				->setWidth($shipment['width'] / 10)
				->setHeight($shipment['height'] / 10)
				->setWeight($shipment['weight'] / 1000)
				->setProductId($shipment['product_id'])
				->setItemReference($order->get_id() .'#' . $index);

			if (isset($data['has_dangerous_goods']) && $data['has_dangerous_goods'] === true) {
				$priceItem->setContainsDangerousGoods(true);
			}

			$priceItem = $this->addProductIds($priceItem, $shipment);

			$priceItem = $this->addSignatureOnDelivery($priceItem, $shipment);


			if ($shipment['extra_cover'] > 0) {
				$extra_cover_feature = new Price_Feature();
				$extra_cover_feature->setCode('TRANSIT_COVER');
				$extra_cover_feature->setAttributes(['cover_amount' => round(floatval($shipment['extra_cover']), 2)]);
				$priceItem->addFeature($extra_cover_feature);
				if ($shipment['extra_cover'] > 500 && $shipment['signature_on_delivery'] === false) {
					$priceItem = $this->addSignatureOnDelivery($priceItem, $shipment, true);
				}
			}

			$priceRequest->addItem( $priceItem );
		}

		$prices = (new Price(Utilities::get_api_connector(), $priceRequest))->getItemPrices(true);

		if (count($prices) === 0) {
			$this->abort('The API did not return any prices, please adjust the inputs and try again.');
		}

		if (isset($prices['errors'])) {
			$this->abort($prices['errors']);
		}

		$prices_by_product_id = $this->filter_by_product_id($data['shipments'], $prices);

		if (isset($prices_by_product_id['errors'])) {
			$this->abort($prices_by_product_id['errors']);
		}

		if (count($prices_by_product_id) > 0) {
			return $prices_by_product_id;
		}

		if (isset($prices['warnings'])) {
			$this->abort($prices['warnings']);
		}

		if (isset($prices['errors'])) {
			$this->abort($prices['errors']);
		}
		return [];
	}

	public function action() {
		return 'calculate_prices';
	}

	/**
	 * @param $shipments
	 * @param $prices
	 *
	 * @return array
	 */
	private function filter_by_product_id( $shipments, $prices )
	{
		$filtered_prices = [];
		foreach ($prices as $price_index => $shipment_prices) {
			if(isset($shipment_prices['errors']) && count($shipment_prices['errors']) > 0) {
				$filtered_prices['errors'][] = $shipment_prices['errors'][0];
			}
			$shipment_index = explode('#', $price_index);
			if (!isset($shipment_index[1])) continue;
			$shipment_index = $shipment_index[1];
			foreach ($shipment_prices as $shipment_price) {
				if (!isset($shipments[$shipment_index])) continue;
				if (!isset($shipment_price['product_id'])) continue;
				if ($shipments[$shipment_index]['product_id'] === $shipment_price['product_id']) {
					$filtered_prices[$shipment_index] = $shipment_price;
				}
			}
		}

		return $filtered_prices;
	}


}
