<?php

namespace AustraliaPost\Extensions\Business\Endpoints;

use AustraliaPost\Extensions\Business\API\Business;
use AustraliaPost\Extensions\Business\API\Wrapper\Create_Order_Request;
use AustraliaPost\Extensions\Business\API\Wrapper\Create_Pickup_Request;
use AustraliaPost\Extensions\Business\API\Wrapper\Pay_For_Order_Request;
use AustraliaPost\Extensions\Business\Helpers\Utilities;

class Create_Order_From_Shipments_Endpoint extends Abstract_Endpoint {

	const OPTION_KEY_ORDERED_SHIPMENTS = '_wpruby_labels_pro_ordered_shipments';
	const OPTION_KEY_ORDER = '_wpruby_labels_pro_order_';

	/**
	 * @param array $data
	 *
	 * @return array|void
	 */
	public function callback( $data )
	{
		$this->validate($data);

		$shipments = $data['shipments'];
		$order_reference = 'order-' . time();

		$create_order_request = new Create_Order_Request();

		$pickup = [];
		if (isset($data['with_pickup']) && $data['with_pickup']) {
			$pickup = $this->createPickupOrder($data);
			if (isset($pickup['errors'])) {
				return $pickup;
			}
		}

		$create_order_request->setOrderReference($order_reference);

		$payment_method = Utilities::get_method_setting('payment_method');
		if (Utilities::isEParcel() || (Utilities::isMyPost() && $payment_method && $payment_method !== 'stored')) {
			$create_order_request->setPaymentMethod(Create_Order_Request::PAYMENT_METHOD_CHARGE_ACCOUNT);
		} else {
			$create_order_request->setPaymentMethod(Create_Order_Request::PAYMENT_METHOD_STORED_PAYMENT);
		}

		foreach ($shipments as $shipment) {
			$create_order_request->addShipment($shipment);
		}

		if (isset($pickup['pickups']) && count($pickup['pickups']) > 0) {
			$create_order_request->addPickupId($pickup['pickups'][0]['pickup_id']);
		}

		$order = (new Business())->create_order($create_order_request);

		if (!isset($order['order'])) {
			return $order;
		}

		if ($create_order_request->getPaymentMethod() === Create_Order_Request::PAYMENT_METHOD_CHARGE_ACCOUNT && Utilities::isMyPost()) {
			$payment = $this->createPaymentWithChargeAccount($order['order']);
			if (isset($payment['errors'])) {
				$order['order']['payment_failure'] = $payment['errors'][0]['message'];
			}
		}

		$this->store_response($order['order'], $order_reference);
		$this->mark_shipments_as_ordered($shipments);

		return $order['order'];

	}

	/**
	 * @return string
	 */
	public function action()
	{
		return 'create_order_from_shipments';
	}

	private function validate( array $data )
	{
		if (!isset($data['shipments'])) {
			$this->abort(['shipments are not provided']);
		}

		if (count($data['shipments']) > 2000) {
			$this->abort('You can not create an order with more than 2000 items. Please, select less shipments.');
		}
	}

	private function store_response( $order, $order_reference ) {
		update_option(self::OPTION_KEY_ORDER . $order_reference , $order, false);
	}

	/**
	 * @param array $shipments
	 */
	private function mark_shipments_as_ordered( $shipments ) {
		$printed_shipments = get_option(self::OPTION_KEY_ORDERED_SHIPMENTS);
		if (!is_array($printed_shipments)) {
			$printed_shipments = [];
		}
		$printed_shipments = array_merge($printed_shipments, $shipments);
		update_option(self::OPTION_KEY_ORDERED_SHIPMENTS, $printed_shipments);
	}

	/**
	 * @param $data
	 * @return array
	 */
	private function createPickupOrder( $data )
	{
		$request = new Create_Pickup_Request();
		foreach ($data['shipments'] as $shipment) {
			$request->addShipment($shipment);
		}
		$request->setPickupDate($data['pickup_date']);
		$request->setProductId($data['pickup_product_id']);
		$request->setFrom($this->get_from_address());

		return (new Business())->create_pickup($request);
	}

	/**
	 * @param array $order
	 *
	 * @return array
	 */
	private function createPaymentWithChargeAccount( $order )
	{
		$request = new Pay_For_Order_Request();
		$request->setPaymentMethod(Create_Order_Request::PAYMENT_METHOD_CHARGE_ACCOUNT)
				->setOrderId($order['order_id'])
				->setAccountNumber(Utilities::get_method_setting('payment_method'));

		return (new Business())->create_payment($request);
	}

}
